/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup POSIF
  * @{
  */

/**
  * @addtogroup POSIF_HALLOutput
  * @{
  */

/* Private typedef --------------------------------------------------------------*/
POSIF_InitTypeStruct POSIF_InitStructType;

/* Private define ---------------------------------------------------------------*/
POSIF_InitTypeStruct POSIF_InitStructType;
uint32_t width=0 ;
uint16_t int_flag_max_width_cnt=0;

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();

  /* LED Init */
  EVAL_LedInit(LED1);
  EVAL_LedInit(LED2);
  EVAL_LedInit(LED3);

  /* Print Uart Init */
  EVAL_ComInit();

  /********************* Operation  *********************/
  /* Enable clock */
  RCC_Unlock();
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_POSIF, ENABLE);
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_GPIO, ENABLE);
  RCC_Lock();
  /* Config posif GPIO */
  GPIO_InitTypeStruct POSIF_GPIO_InitStruct;
  POSIF_GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  POSIF_GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  POSIF_GPIO_InitStruct.GPIO_OType = GPIO_OTYPE_PP;
  POSIF_GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_NOPULL;
  POSIF_GPIO_InitStruct.GPIO_Pin = GPIO_PIN_2;
  GPIO_Init(GPIOA, &POSIF_GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOA, GPIO_PIN_2, GPIO_ALT_FUNC_0);
  POSIF_GPIO_InitStruct.GPIO_Pin = GPIO_PIN_3;
  GPIO_Init(GPIOA, &POSIF_GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOA, GPIO_PIN_3, GPIO_ALT_FUNC_0);
  POSIF_GPIO_InitStruct.GPIO_Pin = GPIO_PIN_6;
  GPIO_Init(GPIOA, &POSIF_GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOA, GPIO_PIN_6, GPIO_ALT_FUNC_1);

  /* Config posif */
  /* Config posif */
  POSIF_DeInit();
  POSIF_StructInit(&POSIF_InitStructType);
  POSIF_InitStructType.POSIF_In0FilterLen = POSIF_FLT_LEN1;
  POSIF_InitStructType.POSIF_In1FilterLen = POSIF_FLT_LEN1;
  POSIF_InitStructType.POSIF_In2FilterLen = POSIF_FLT_LEN1;
  POSIF_InitStructType.POSIF_In0ClockSample = POSIF_FLT_SAMPLE1;
  POSIF_InitStructType.POSIF_In1ClockSample = POSIF_FLT_SAMPLE1;
  POSIF_InitStructType.POSIF_In2ClockSample = POSIF_FLT_SAMPLE1;
  POSIF_InitStructType.POSIF_In0AnalogCompareSource = POSIF_ACMP_ACMP0;
  POSIF_InitStructType.POSIF_In1AnalogCompareSource = POSIF_ACMP_ACMP0;
  POSIF_InitStructType.POSIF_In2AnalogCompareSource = POSIF_ACMP_ACMP0;
  POSIF_InitStructType.POSIF_In0InputSource = POSIF_SRC_PIN;
  POSIF_InitStructType.POSIF_In1InputSource = POSIF_SRC_PIN;
  POSIF_InitStructType.POSIF_In2InputSource = POSIF_SRC_PIN;
  POSIF_InitStructType.POSIF_Mode = POSIF_MODE_HALL;
  POSIF_InitStructType.POSIF_ClockDiv = POSIF_CLK_DIV32;
  POSIF_InitStructType.HALL_MaxWidth = 5000;
  POSIF_Init(&POSIF_InitStructType);
  POSIF_Cmd(ENABLE);

  DelayMs(2);  
    
  /* Get width value */
  width = POSIF_HALL_GetWidth();
  
  /* Enable interrupt */
  int_flag_max_width_cnt=0;
  NVIC_SetPriority(POSIF_IRQn, 2);
  NVIC_EnableIRQ(POSIF_IRQn);
  POSIF_HALL_IntCmd(HALL_INT_TYPE_MAX_WIDTH, ENABLE);

  uint32_t ledTime = 0;
  /* Infinite loop */
  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}


/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
